(define-library (owl sys2)
   (export
      errno
      getcwd ; Get Working Directory
      chdir

      uname
      isatty
      getenv)

   (import
      (scheme core)
      (otus ffi)

      (owl string)
      (owl math)
      (owl io)
      (otus async)
      (owl list))

(cond-expand
   (Linux
      (begin
         (setq SO (or
            (load-dynamic-library "libc.so")
            (load-dynamic-library "libc.so.6")))

         (setq :getcwd (SO type-string "getcwd" type-string fft-int))
         (setq :chdir (SO fft-int "chdir" type-string))
      ))
   (Windows
      (begin
         (setq SO (or
            (load-dynamic-library "msvcr71.dll")  ; vcrun2003
            (load-dynamic-library "msvcr80.dll")  ; vcrun2005
            (load-dynamic-library "msvcr90.dll")  ; vcrun2008
            (load-dynamic-library "msvcr100.dll") ; vcrun2010
            (load-dynamic-library "msvcr110.dll") ; vcrun2012
            (load-dynamic-library "msvcr120.dll") ; vcrun2013
            (load-dynamic-library "ucrtbase.dll") ; vcrun2015, vcrun2017, vcrun2019, ...
            (runtime-error "No Visual C++ Redistributables are found. Please, install at least one.")))

         (setq :getcwd (SO type-string-wide "_wgetcwd" type-string-wide fft-int))
         (setq :chdir (SO fft-int "_wchdir" type-string-wide))
      ))
   (Darwin
      (begin
         (setq SO
            (load-dynamic-library "libSystem.dylib"))

         (setq :getcwd (SO type-string "getcwd" type-string fft-int))
         (setq :chdir (SO fft-int "chdir" type-string))
      ))
   (else
      (begin
         (runtime-error "Unsupported platform" *uname*) )))

(begin
   (define (errno) (syscall 60))

   (define (getcwd)
      (let loop ((n 128))
         (define cwd (:getcwd (make-string n 0) n))
         (if cwd cwd (loop (+ n 64)))))

   (define chdir :chdir)


   (define (uname)
      (syscall 63))
   (define (isatty fd)
      (syscall 16 fd 19))

   (define (getenv str)
      (let ((str (c-string str)))
         (if str
            (syscall 1016 str))))
))
